#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## conf_snmp.pl for configuration automatique snmp
## 
## ######### PROJECT DESCRIPTION : ###
## 
## configuration automatique du demon snmp
## 
## ###################################


## ###################################
## Librairies

use	strict;

use	Getopt::Long;		# paramtres ligne de commande
use	raid;

use	Data::Dumper;

## ###################################
## Declaration de variables globales

# Variable option
my	$help		= "";
my	$clean		= "";
my	$test		= "";
my	$write		= "";

# chemin du fichier de conf snmp
my	$path_conf	= "/etc/snmp/snmpd.conf";
# OID general
my	$oid		= "1.3.6.1.4.1.2021.4242.";


## ###################################
## Recuperation des arguments

sub	getCLOptions {
    Getopt::Long::Configure('noignore_case');
    unless (
        GetOptions(
            'help|h'    => \$help,
            'clean'   => \$clean,
            'test'    => \$test,
            'write'   => \$write
        )
    )
    {
        die "Abandon : option incorrecte\n";
    }
}

## ###################################
## Print Help

sub	print_help {

    print <<EOF;
	
    Nom du programme  : $0 - Configuration automatique snmp
    Description	      : Configuration de /etc/snmp/snmpd.conf
	
    Options:
        --help      Affiche cette rubrique d\'aide
        --clean     Nettoie le fichier de configuration snmp
        --test      Test seulement, n\'ecrit pas la configuration
        --write     Lance l\'ecriture de $path_conf

    Test			:
        export MIBS=+STORIQ-TABLE-MIB
        snmptranslate -IR -Tp -On storiq
        ...
        snmpwalk -On -v 1 -c public localhost .1.3.6.1.4.1.2021.4242

    Copyright (c) Intellique 2007
    All rights reserved.
EOF

    exit 1;
}

sub	raid_calling {

    my ($etat, $ref_info) = raid::info();
    
    if ( $etat ) {
        if ( ref( $ref_info ) eq "HASH" ) {
            # Il y a des infos  afficher
            print "ERROR : ". $ref_info->{Erreur} ."\n";
            exit 0;
        } else {
            # Il y a du texte  afficher
            print "ERROR : $ref_info\n";
            exit 0;
        }
    }
    
    my $ctl_nb = 0;
    my @tab;
    
    foreach my $ctl ( sort keys %{ $ref_info } ) {
        $ctl_nb++;
	
		# +-- -R-- String    ControllerType(1)
		push(@tab, "exec .".$oid."1.".$ctl_nb." = /bin/echo logiciel") if ($ctl =~ /^clo/i);
		push(@tab, "exec .".$oid."1.".$ctl_nb." = /bin/echo materiel") if ($ctl =~ /^cm/i);
	
		# +-- -R-- String    ControllerNbDrives(2)
		push(@tab, "exec .".$oid."2.".$ctl_nb." = /bin/echo ".$ref_info->{$ctl}->{'Drives'});
	
		# +-- -R-- String    ControllerNbPorts(4)
		push(@tab, "exec .".$oid."4.".$ctl_nb." = /bin/echo ".$ref_info->{$ctl}->{'Ports'});
	
		# +-- -R-- String    ControllerNbUnits(5)
		push(@tab, "exec .".$oid."5.".$ctl_nb." = /bin/echo ".$ref_info->{$ctl}->{'Units'});
	
		# +-- -R-- String    ControllerName(6)
		push(@tab, "exec .".$oid."6.".$ctl_nb." = /bin/echo ".$ctl);
	
		
		# +-- -R-- String    ControllerManufacturer(7)
		push(@tab, "exec .".$oid."7.".$ctl_nb." = /bin/echo mdadm") if ($ctl =~ /clo/i);
		push(@tab, "exec .".$oid."7.".$ctl_nb." = /bin/echo ".$1) if ($ctl =~ /cm_([0-9a-z]*)_.*/i);
	
	
		# +-- -R-- String    ControllerModel(8)
		push(@tab, "exec .".$oid."8.".$ctl_nb." = /bin/echo ".$ref_info->{$ctl}->{'Model'});
	
		# +-- -R-- String    ControllerNotOpt(3)
        push(@tab, "exec .". $oid ."3.$ctl_nb /usr/bin/perl /usr/local/bin/raid_control -f nagios $ctl");
	
		my $unit_nb = 0;
        foreach my $unit (sort keys %{ $ref_info->{$ctl}{'Unit'} } ) {
		    $unit_nb++;
		    
		    # +-- -R-- String    UnitType(9)
		    push(@tab, "exec .".$oid."9.".$ctl_nb.".".$unit_nb." = /bin/echo ".$ref_info->{$ctl}->{'Unit'}->{$unit}->{'UnitType'} );
		    
		    # +-- -R-- String    UnitName(10)
		    push(@tab, "exec .".$oid."10.".$ctl_nb.".".$unit_nb." = /bin/echo ".$unit);
		    
		    # +-- -R-- String    UnitController(11)
		    push(@tab, "exec .".$oid."11.".$ctl_nb.".".$unit_nb." = /bin/echo ".$ctl);
		    
		    # +-- -R-- String    UnitStatus(12)
            push(@tab, "exec .". $oid ."12.$ctl_nb.$unit_nb /usr/bin/perl /usr/local/bin/raid_control -f nagios $ctl $unit");
		    
		    # +-- -R-- String    UnitSize(13)
		    push(@tab, "exec .".$oid."13.".$ctl_nb.".".$unit_nb." = /bin/echo ".$ref_info->{$ctl}->{'Unit'}->{$unit}->{'Size'});
		}

		my $port_nb = 0;
		foreach my $port (sort keys( %{$ref_info->{$ctl}{'Drive'}})) {
		    $port_nb++;
		    
		    # +-- -R-- String    PortType(14)
		    push(@tab, "exec .".$oid."14.".$ctl_nb.".".$port_nb." = /bin/echo ".$ref_info->{$ctl}->{'Drive'}->{$port}->{'Model'});
	
		    # +-- -R-- String    PortController(16)
		    push(@tab, "exec .".$oid."16.".$ctl_nb.".".$port_nb." = /bin/echo ".$ctl);
	
		    # +-- -R-- String    PortName(15)
		    push(@tab, "exec .".$oid."15.".$ctl_nb.".".$port_nb." = /bin/echo ".$port);
	
		    # +-- -R-- String    PortSerial(19)
		    push(@tab, "exec .".$oid."19.".$ctl_nb.".".$port_nb." = /bin/echo ".$ref_info->{$ctl}->{'Drive'}->{$port}->{'Serial'});
	
		    # +-- -R-- String    PortManufacturer(20)
		    push(@tab, "exec .".$oid."20.".$ctl_nb.".".$port_nb." = /bin/echo ".$ref_info->{$ctl}->{'Drive'}->{$port}->{'Model'});
	
		    # +-- -R-- String    PortSize(21)
		    push(@tab, "exec .".$oid."21.".$ctl_nb.".".$port_nb." = /bin/echo ".$ref_info->{$ctl}->{'Drive'}->{$port}->{'Size'});
	
		    # +-- -R-- String    PortStatus(17)
            push(@tab, "exec .". $oid ."17.$ctl_nb.$port_nb /usr/bin/perl /usr/local/bin/raid_control -f nagios $ctl $port");
		    
		    # +-- -R-- String    PortUnit(18)
		    push(@tab, "exec .".$oid."18.".$ctl_nb.".".$port_nb." = /bin/echo ".$ref_info->{$ctl}->{'Drive'}->{$port}->{'Unit'});
		}

    }
    #print Dumper($ref_info);
    #print Dumper (\@tab);
    return @tab;
}

## ###################################
## Nettoyage de la configuration snmp
## (Suppression des ajouts Storiq)

sub	cleaning {
    unless (-f $path_conf) {
        print "File $path_conf not found.\n";
   	    exit 1;
    }
    
    print "Nettoyage de $path_conf...";
    
    qx#mv $path_conf $path_conf.old#;

    open (FH, $path_conf.".old") or die ($!);
    open (NEW, ">".$path_conf) or die ($!);

    while (<FH>) {
        unless ($_ =~ m/$oid/i) {
            print NEW $_;
        }
    }

    close (NEW);
    close (FH);

    print  "OK.\n";
   	exit 0;
}

## ###################################
## Ecriture du fichier de configuration

sub	writing {

    foreach ( @_ ) {
        qx#echo $_ >> $path_conf#;
    }

    print "OK.\n";
	exit 0;
}

## ###################################
## Test du fichier de configuration

sub	testing {

    foreach (@_) {
        print "\tcmd\t: $_\n";
    }

	exit 0;
}

## ###################################
## Main

## Recuperation des arguments, sinon affichage de l'aide 
if (scalar(@ARGV)) {
    getCLOptions();
}

print_help() if ( $help );

cleaning() if ( $clean );

my @tab = raid_calling();

testing( @tab ) if ($test);
writing( @tab ) if ($write);

print_help();

exit 0;


## ###################################
## fin
## ###################################


## ###################################
## Appel de la table de hash raid
# +--storiq(4242)
#    |
#    +--RaidStoriq(1)
#       |
#       +-- -R-- String    ControllerType(1)
#       +-- -R-- String    ControllerNbDrives(2)
#       +-- -R-- String    ControllerNotOpt(3)
#       +-- -R-- String    ControllerNbPorts(4)
#       +-- -R-- String    ControllerNbUnits(5)
#       +-- -R-- String    ControllerName(6)
#       +-- -R-- String    ControllerManufacturer(7)
#       +-- -R-- String    ControllerModel(8)
#       +-- -R-- String    UnitType(9)
#       +-- -R-- String    UnitName(10)
#       +-- -R-- String    UnitController(11)
#       +-- -R-- String    UnitStatus(12)
#       +-- -R-- String    UnitSize(13)
#       +-- -R-- String    PortType(14)
#       +-- -R-- String    PortName(15)
#       +-- -R-- String    PortController(16)
#       +-- -R-- String    PortStatus(17)
#       +-- -R-- String    PortUnit(18)
#       +-- -R-- String    PortSerial(19)
#       +-- -R-- String    PortManufacturer(20)
#       +-- -R-- String    PortSize(21)
